@@ -26,6 +26,7 @@ pub fn create_payload(
     sub_command: Option<SubCommand>,
     template_info: Option<TemplateInfo>,
     timeout_milliseconds: Option<u16>,
+    use_pre_bio_enrollment: bool
 ) -> Vec<u8> {
     let mut map = BTreeMap::new();
 
@@ -84,7 +85,11 @@ pub fn create_payload(
     let cbor = Value::Map(map);
 
     // create payload
-    let mut payload = [ctapdef::AUTHENTICATOR_BIO_ENROLLMENT].to_vec();
+    let mut payload = if use_pre_bio_enrollment {
+        [ctapdef::AUTHENTICATOR_BIO_ENROLLMENT_P].to_vec()
+    } else {
+        [ctapdef::AUTHENTICATOR_BIO_ENROLLMENT].to_vec()
+    };
     payload.append(&mut to_vec(&cbor).unwrap());
     payload
 }
