@@ -2,8 +2,10 @@ use crate::cose;
 use crate::util;
 use serde_cbor::Value;
 
+#[derive(Default)]
 pub struct Pin {
     pub retries: i32,
+    pub uv_retries: i32,
 }
 
 pub fn parse_cbor_client_pin_get_pin_token(bytes: &[u8]) -> Result<Vec<u8>, String> {
@@ -40,13 +42,14 @@ pub fn parse_cbor_client_pin_get_retries(bytes: &[u8]) -> Result<Pin, String> {
     // deserialize to a serde_cbor::Value
     let cbor: Value = serde_cbor::from_slice(bytes).unwrap();
 
-    let mut pin = Pin { retries: 0 };
+    let mut pin = Pin::default();
 
     if let Value::Map(n) = cbor {
         for (key, val) in &n {
             if let Value::Integer(member) = key {
                 match member {
                     3 => pin.retries = util::cbor_value_to_num(val)?,
+                    5 => pin.uv_retries = util::cbor_value_to_num(val)?,
                     _ => println!("- anything error"),
                 }
             }
