@@ -25,10 +25,10 @@ impl fmt::Display for CoseKey {
             strbuf.append("- crv", &intval);
         }
         if let Some(Value::Bytes(bytes)) = self.parameters.get(&-2) {
-            strbuf.appenh("- x", &bytes);
+            strbuf.appenh("- x", bytes);
         }
         if let Some(Value::Bytes(bytes)) = self.parameters.get(&-3) {
-            strbuf.appenh("- y", &bytes);
+            strbuf.appenh("- y", bytes);
         }
         write!(f, "{}", strbuf.build())
     }
