@@ -6,26 +6,25 @@ use crate::ctaphid;
 use crate::public_key_credential_descriptor::PublicKeyCredentialDescriptor;
 use crate::public_key_credential_user_entity::PublicKeyCredentialUserEntity;
 use crate::FidoKeyHid;
-use crate::HidParam;
 
 #[allow(unused_imports)]
 use crate::util;
+use anyhow::{Error, Result};
 
 pub(crate) fn credential_management(
-    hid_params: &[HidParam],
+    device: &FidoKeyHid,
     pin: Option<&str>,
     sub_command: credential_management_command::SubCommand,
     rpid_hash: Option<Vec<u8>>,
     pkcd: Option<PublicKeyCredentialDescriptor>,
     pkcue: Option<PublicKeyCredentialUserEntity>,
-) -> Result<credential_management_params::CredentialManagementData, String> {
-    let device = FidoKeyHid::new(hid_params)?;
-    let cid = ctaphid::ctaphid_init(&device)?;
+) -> Result<credential_management_params::CredentialManagementData> {
+    let cid = ctaphid::ctaphid_init(device).map_err(Error::msg)?;
 
     // pin token
     let pin_token = {
         if let Some(pin) = pin {
-            Some(client_pin::get_pin_token(&device, &cid, pin)?)
+            Some(client_pin::get_pin_token(device, &cid, pin)?)
         } else {
             None
         }
@@ -37,16 +36,17 @@ pub(crate) fn credential_management(
         rpid_hash,
         pkcd,
         pkcue,
+        device.use_pre_credential_management,
     );
 
-    if util::is_debug() {
+    if device.enable_log {
         println!("send(cbor) = {}", util::to_hex_str(&send_payload));
     }
 
-    let response_cbor = ctaphid::ctaphid_cbor(&device, &cid, &send_payload)?;
-    if util::is_debug() {
+    let response_cbor = ctaphid::ctaphid_cbor(device, &cid, &send_payload).map_err(Error::msg)?;
+    if device.enable_log {
         println!("response(cbor) = {}", util::to_hex_str(&response_cbor));
     }
 
-    credential_management_response::parse_cbor(&response_cbor)
+    Ok(credential_management_response::parse_cbor(&response_cbor).map_err(Error::msg)?)
 }
