@@ -1,10 +1,10 @@
 use crate::ctapdef;
+use crate::get_assertion_params::Extension;
 use crate::hmac::HmacExt;
 use crate::util;
 use serde_cbor::to_vec;
 use serde_cbor::Value;
 use std::collections::BTreeMap;
-use crate::get_assertion_params::Extension;
 
 #[derive(Debug, Default)]
 pub struct Params {
@@ -71,7 +71,10 @@ pub fn create_payload(params: Params, hmac_ext: Option<HmacExt>) -> Vec<u8> {
             // saltAuth(0x03)
             param.insert(Value::Integer(0x03), Value::Bytes(hmac_ext.salt_auth));
 
-            ext_val.insert(Value::Text(Extension::HmacSecret(None).to_string()), Value::Map(param));
+            ext_val.insert(
+                Value::Text(Extension::HmacSecret(None).to_string()),
+                Value::Map(param),
+            );
 
             Some(Value::Map(ext_val))
         } else {
