@@ -37,17 +37,17 @@ impl fmt::Display for Assertion {
             .append("- user", &self.user)
             .appenh("- credential_id", &self.credential_id);
 
-            for e in &self.extensions {
-                match e {
-                    Extension::HmacSecret(d) => {
-                        if let Some(output1_enc) = d {
-                            let tmp = format!("- {}",Extension::HmacSecret(None).to_string());
-                            strbuf.appenh(&tmp, &output1_enc.to_vec());
-                        }
+        for e in &self.extensions {
+            match e {
+                Extension::HmacSecret(d) => {
+                    if let Some(output1_enc) = d {
+                        let tmp = format!("- {}", Extension::HmacSecret(None).to_string());
+                        strbuf.appenh(&tmp, &output1_enc.to_vec());
                     }
                 }
             }
-                    
+        }
+
         write!(f, "{}", strbuf.build())
     }
 }
@@ -62,7 +62,7 @@ impl Extension {
     pub fn create_hmac_secret_from_string(message: &str) -> Extension {
         let mut salt = [0u8; 32];
         let mut digest = Sha256::new();
-        digest.input(&message.as_bytes());
+        digest.input(message.as_bytes());
         digest.result(&mut salt);
         //print!("{}", StrBuf::bufh("- salt", &salt));
         Extension::HmacSecret(Some(salt))
